----------------------------------------------------------------------------------------------
--
--
--      Input file         : C:/Download/_Halbleiter/Altera/terasic/DE0_Nano_v1.0.5_SystemCD/Demonstration/DE0_NANO_GSensor.vhdl/v/spi_controller.v
--      Design name        : spi_controller
--      Author             : 
--      Company            : 
--
--      Description        : 
--
--
----------------------------------------------------------------------------------------------
--
LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.std_logic_misc.all;

ENTITY spi_controller IS
   GENERIC (
      -- Data MSB Bit
      IDLE_MSB                       :  integer := 14;    
      SI_DataL                       :  integer := 15;    
      SO_DataL                       :  integer := 7;    
      -- Write/Read Mode 
      WRITE_MODE                     :  std_logic_vector(1 DOWNTO 0) := "00";    
      READ_MODE                      :  std_logic_vector(1 DOWNTO 0) := "10";    
      -- Initial Reg Number 
      INI_NUMBER                     :  std_logic_vector(3 DOWNTO 0) := "1011";    
      -- SPI State 
      IDLE                           :  std_logic := '0';    
      TRANSFER                       :  std_logic := '1';    
      -- Write Reg Address 
      BW_RATE                        :  std_logic_vector(5 DOWNTO 0) := "101100";    
      POWER_CONTROL                  :  std_logic_vector(5 DOWNTO 0) := "101101";    
      DATA_FORMAT                    :  std_logic_vector(5 DOWNTO 0) := "110001";    
      INT_ENABLE                     :  std_logic_vector(5 DOWNTO 0) := "101110";    
      INT_MAP                        :  std_logic_vector(5 DOWNTO 0) := "101111";    
      THRESH_ACT                     :  std_logic_vector(5 DOWNTO 0) := "100100";    
      THRESH_INACT                   :  std_logic_vector(5 DOWNTO 0) := "100101";    
      TIME_INACT                     :  std_logic_vector(5 DOWNTO 0) := "100110";    
      ACT_INACT_CTL                  :  std_logic_vector(5 DOWNTO 0) := "100111";    
      THRESH_FF                      :  std_logic_vector(5 DOWNTO 0) := "101000";    
      TIME_FF                        :  std_logic_vector(5 DOWNTO 0) := "101001";    
      -- Read Reg Address
      INT_SOURCE                     :  std_logic_vector(5 DOWNTO 0) := "110000";    
      X_LB                           :  std_logic_vector(5 DOWNTO 0) := "110010";    
      X_HB                           :  std_logic_vector(5 DOWNTO 0) := "110011";    
      Y_LB                           :  std_logic_vector(5 DOWNTO 0) := "110100";    
      Y_HB                           :  std_logic_vector(5 DOWNTO 0) := "110101";    
      Z_LB                           :  std_logic_vector(5 DOWNTO 0) := "110110";    
      Z_HB                           :  std_logic_vector(5 DOWNTO 0) := "110111");    
   PORT (
      iRSTN                   : IN std_logic;   
      iSPI_CLK                : IN std_logic;   
      iSPI_CLK_OUT            : IN std_logic;   
      iP2S_DATA               : IN std_logic_vector(SI_DataL DOWNTO 0);   
      iSPI_GO                 : IN std_logic;   
      oSPI_END                : OUT std_logic;   
      oS2P_DATA               : OUT std_logic_vector(SO_DataL DOWNTO 0);   
      SPI_SDIO                : INOUT std_logic;   
      oSPI_CSN                : OUT std_logic;   
      oSPI_CLK                : OUT std_logic);   
END spi_controller;

ARCHITECTURE translated OF spi_controller IS


   --=======================================================
   --  REG/WIRE declarations
   --=======================================================
   SIGNAL read_mode_xhdl5          :  std_logic;   
   SIGNAL write_address            :  std_logic;   
   SIGNAL spi_count_en             :  std_logic;   
   SIGNAL spi_count                :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL temp_xhdl6               :  std_logic;   
   SIGNAL temp_xhdl7               :  std_logic;   
   SIGNAL oSPI_END_xhdl1           :  std_logic;   
   SIGNAL oS2P_DATA_xhdl2          :  std_logic_vector(SO_DataL DOWNTO 0);   
   SIGNAL oSPI_CSN_xhdl3           :  std_logic;   
   SIGNAL oSPI_CLK_xhdl4           :  std_logic;   

BEGIN
   oSPI_END <= oSPI_END_xhdl1;
   oS2P_DATA <= oS2P_DATA_xhdl2;
   oSPI_CSN <= oSPI_CSN_xhdl3;
   oSPI_CLK <= oSPI_CLK_xhdl4;
   read_mode_xhdl5 <= iP2S_DATA(SI_DataL) ;
   write_address <= spi_count(3) ;
   oSPI_END_xhdl1 <= NOR_REDUCE(spi_count) ;
   oSPI_CSN_xhdl3 <= NOT iSPI_GO ;
   temp_xhdl6 <= iSPI_CLK_OUT WHEN spi_count_en = '1' ELSE '1';
   oSPI_CLK_xhdl4 <= temp_xhdl6 ;
   temp_xhdl7 <= iP2S_DATA(conv_integer(spi_count)) WHEN (spi_count_en AND (NOT read_mode_xhdl5 OR write_address)) = '1' ELSE 'Z';
   SPI_SDIO <= temp_xhdl7 ;

   PROCESS (iRSTN, iSPI_CLK)
   BEGIN
      IF (iRSTN = '0') THEN
         spi_count_en <= '0';    
         spi_count <= "1111";    
      ELSIF rising_edge(iSPI_CLK) THEN
          IF (oSPI_END_xhdl1 = '1') THEN
            spi_count_en <= '0';    
         ELSE
            IF (iSPI_GO = '1') THEN
               spi_count_en <= '1';    
            END IF;
         END IF;
         IF (NOT spi_count_en = '1') THEN
            spi_count <= "1111";    
         ELSE
            spi_count <= spi_count - "0001";    
         END IF;
         IF ((read_mode_xhdl5 AND NOT write_address) = '1') THEN
            oS2P_DATA_xhdl2 <= oS2P_DATA_xhdl2(SO_DataL - 1 DOWNTO 0) & SPI_SDIO;    
         END IF;
      END IF;
   END PROCESS;

END translated;
