----------------------------------------------------------------------------------------------
--
--
--      Input file         : C:/Download/_Halbleiter/Altera/terasic/DE0_Nano_v1.0.5_SystemCD/Demonstration/DE0_NANO_GSensor.vhdl/v/spi_ee_config.v
--      Design name        : spi_ee_config
--      Author             : 
--      Company            : 
--
--      Description        : 
--
--
----------------------------------------------------------------------------------------------
--
LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

ENTITY spi_ee_config IS
   GENERIC (
      -- Data MSB Bit
      IDLE_MSB                       :  integer := 14;    
      SI_DataL                       :  integer := 15;    
      SO_DataL                       :  integer := 7;    
      -- Write/Read Mode 
      WRITE_MODE                     :  std_logic_vector(1 DOWNTO 0) := "00";    
      READ_MODE                      :  std_logic_vector(1 DOWNTO 0) := "10";    
      -- Initial Reg Number 
      INI_NUMBER                     :  std_logic_vector(3 DOWNTO 0) := "1011";    
      -- SPI State 
      IDLE                           :  std_logic := '0';    
      TRANSFER                       :  std_logic := '1';    
      -- Write Reg Address 
      BW_RATE                        :  std_logic_vector(5 DOWNTO 0) := "101100";    
      POWER_CONTROL                  :  std_logic_vector(5 DOWNTO 0) := "101101";    
      DATA_FORMAT                    :  std_logic_vector(5 DOWNTO 0) := "110001";    
      INT_ENABLE                     :  std_logic_vector(5 DOWNTO 0) := "101110";    
      INT_MAP                        :  std_logic_vector(5 DOWNTO 0) := "101111";    
      THRESH_ACT                     :  std_logic_vector(5 DOWNTO 0) := "100100";    
      THRESH_INACT                   :  std_logic_vector(5 DOWNTO 0) := "100101";    
      TIME_INACT                     :  std_logic_vector(5 DOWNTO 0) := "100110";    
      ACT_INACT_CTL                  :  std_logic_vector(5 DOWNTO 0) := "100111";    
      THRESH_FF                      :  std_logic_vector(5 DOWNTO 0) := "101000";    
      TIME_FF                        :  std_logic_vector(5 DOWNTO 0) := "101001";    
      -- Read Reg Address
      INT_SOURCE                     :  std_logic_vector(5 DOWNTO 0) := "110000";    
      X_LB                           :  std_logic_vector(5 DOWNTO 0) := "110010";    
      X_HB                           :  std_logic_vector(5 DOWNTO 0) := "110011";    
      Y_LB                           :  std_logic_vector(5 DOWNTO 0) := "110100";    
      Y_HB                           :  std_logic_vector(5 DOWNTO 0) := "110101";    
      Z_LB                           :  std_logic_vector(5 DOWNTO 0) := "110110";    
      Z_HB                           :  std_logic_vector(5 DOWNTO 0) := "110111");    
   PORT (
      iRSTN                   : IN std_logic;   
      iSPI_CLK                : IN std_logic;   
      iSPI_CLK_OUT            : IN std_logic;   
      iG_INT2                 : IN std_logic;   
      oDATA_L                 : OUT std_logic_vector(SO_DataL DOWNTO 0);   
      oDATA_H                 : OUT std_logic_vector(SO_DataL DOWNTO 0);   
      SPI_SDIO                : INOUT std_logic;   
      oSPI_CSN                : OUT std_logic;   
      oSPI_CLK                : OUT std_logic);   
END spi_ee_config;

ARCHITECTURE translated OF spi_ee_config IS

   COMPONENT spi_controller
      GENERIC (
          X_LB                           :  std_logic_vector(5 DOWNTO 0) := "110010";    
          Z_HB                           :  std_logic_vector(5 DOWNTO 0) := "110111";    
         -- Data MSB Bit
         -- Initial Reg Number 
          Y_LB                           :  std_logic_vector(5 DOWNTO 0) := "110100";    
          READ_MODE                      :  std_logic_vector(1 DOWNTO 0) := "10";    
          Z_LB                           :  std_logic_vector(5 DOWNTO 0) := "110110";    
          SI_DataL                       :  integer := 15;    
          THRESH_ACT                     :  std_logic_vector(5 DOWNTO 0) := "100100";    
          THRESH_INACT                   :  std_logic_vector(5 DOWNTO 0) := "100101";    
          POWER_CONTROL                  :  std_logic_vector(5 DOWNTO 0) := "101101";    
          SO_DataL                       :  integer := 7;    
          INT_ENABLE                     :  std_logic_vector(5 DOWNTO 0) := "101110";    
          THRESH_FF                      :  std_logic_vector(5 DOWNTO 0) := "101000";    
         -- SPI State 
          TIME_FF                        :  std_logic_vector(5 DOWNTO 0) := "101001";    
          TIME_INACT                     :  std_logic_vector(5 DOWNTO 0) := "100110";    
          TRANSFER                       :  std_logic := '1';    
          ACT_INACT_CTL                  :  std_logic_vector(5 DOWNTO 0) := "100111";    
          DATA_FORMAT                    :  std_logic_vector(5 DOWNTO 0) := "110001";    
          X_HB                           :  std_logic_vector(5 DOWNTO 0) := "110011";    
         -- Write/Read Mode 
         -- Read Reg Address
         -- Write Reg Address 
          INT_MAP                        :  std_logic_vector(5 DOWNTO 0) := "101111";    
          Y_HB                           :  std_logic_vector(5 DOWNTO 0) := "110101");    
      PORT (
         iRSTN                   : IN  std_logic;
         iSPI_CLK                : IN  std_logic;
         iSPI_CLK_OUT            : IN  std_logic;
         iP2S_DATA               : IN  std_logic_vector(SI_DataL DOWNTO 0);
         iSPI_GO                 : IN  std_logic;
         oSPI_END                : OUT std_logic;
         oS2P_DATA               : OUT std_logic_vector(SO_DataL DOWNTO 0);
         SPI_SDIO                : INOUT std_logic;
         oSPI_CSN                : OUT std_logic;
         oSPI_CLK                : OUT std_logic);
   END COMPONENT;


   --=======================================================
   --  REG/WIRE declarations
   --=======================================================
   SIGNAL ini_index                :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL write_data               :  std_logic_vector(SI_DataL - 2 DOWNTO 0);   
   SIGNAL p2s_data                 :  std_logic_vector(SI_DataL DOWNTO 0);   
   SIGNAL spi_go                   :  std_logic;   
   SIGNAL spi_end                  :  std_logic;   
   SIGNAL s2p_data                 :  std_logic_vector(SO_DataL DOWNTO 0);   
   SIGNAL low_byte_data            :  std_logic_vector(SO_DataL DOWNTO 0);   
   SIGNAL spi_state                :  std_logic;   
   SIGNAL high_byte                :  std_logic;   
   SIGNAL read_back                :  std_logic;   
   SIGNAL clear_status             :  std_logic;   
   SIGNAL read_ready               :  std_logic;   
   SIGNAL clear_status_d           :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL high_byte_d              :  std_logic;   
   SIGNAL read_back_d              :  std_logic;   
   SIGNAL read_idle_count          :  std_logic_vector(IDLE_MSB DOWNTO 0);   
   SIGNAL oDATA_L_xhdl1            :  std_logic_vector(SO_DataL DOWNTO 0);   
   SIGNAL oDATA_H_xhdl2            :  std_logic_vector(SO_DataL DOWNTO 0);   
   SIGNAL oSPI_CSN_xhdl3           :  std_logic;   
   SIGNAL oSPI_CLK_xhdl4           :  std_logic;   

BEGIN
   oDATA_L <= oDATA_L_xhdl1;
   oDATA_H <= oDATA_H_xhdl2;
   oSPI_CSN <= oSPI_CSN_xhdl3;
   oSPI_CLK <= oSPI_CLK_xhdl4;
   u_spi_controller : spi_controller 
      PORT MAP (
         iRSTN => iRSTN,
         iSPI_CLK => iSPI_CLK,
         iSPI_CLK_OUT => iSPI_CLK_OUT,
         iP2S_DATA => p2s_data,
         iSPI_GO => spi_go,
         oSPI_END => spi_end,
         oS2P_DATA => s2p_data,
         SPI_SDIO => SPI_SDIO,
         oSPI_CSN => oSPI_CSN_xhdl3,
         oSPI_CLK => oSPI_CLK_xhdl4);   
   

   PROCESS (ini_index)
   BEGIN
      CASE ini_index IS
         WHEN "0000" =>
                  write_data <= THRESH_ACT & "00100000";    
         WHEN "0001" =>
                  write_data <= THRESH_INACT & "00000011";    
         WHEN "0010" =>
                  write_data <= TIME_INACT & "00000001";    
         WHEN "0011" =>
                  write_data <= ACT_INACT_CTL & "01111111";    
         WHEN "0100" =>
                  write_data <= THRESH_FF & "00001001";    
         WHEN "0101" =>
                  write_data <= TIME_FF & "01000110";    
         WHEN "0110" =>
                  write_data <= BW_RATE & "00001001";    
         WHEN "0111" =>
                  write_data <= INT_ENABLE & "00010000";    
         WHEN "1000" =>
                  write_data <= INT_MAP & "00010000";    
         WHEN "1001" =>
                  write_data <= DATA_FORMAT & "01000000";    
         WHEN OTHERS  =>
                  write_data <= POWER_CONTROL & "00001000";    
         
      END CASE;
   END PROCESS;

   PROCESS (iRSTN, iSPI_CLK)
   BEGIN
      IF (iRSTN = '0') THEN
         ini_index <= "0000";    
         spi_go <= '0';    
         spi_state <= IDLE;    
         read_idle_count <= (OTHERS => '0');    
         high_byte <= '0';    
         read_back <= '0';    
         clear_status <= '0';    
      ELSIF rising_edge(iSPI_CLK) THEN
         IF (ini_index < INI_NUMBER) THEN
            CASE spi_state IS
               WHEN IDLE =>
                        p2s_data <= WRITE_MODE & write_data;    
                        spi_go <= '1';    
                        spi_state <= TRANSFER;    
               WHEN TRANSFER =>
                        IF (spi_end = '1') THEN
                           ini_index <= ini_index + "0001";    
                           spi_go <= '0';    
                           spi_state <= IDLE;    
                        END IF;
               WHEN OTHERS =>
                        NULL;
               
            END CASE;
         ELSE
            CASE spi_state IS
               WHEN IDLE =>
                        read_idle_count <= read_idle_count + "000000000000001";    
                        IF (high_byte = '1') THEN
                           p2s_data(15 DOWNTO 8) <= READ_MODE & X_HB;    
                           read_back <= '1';    
                        ELSE
                           IF (read_ready = '1') THEN
                              p2s_data(15 DOWNTO 8) <= READ_MODE & X_LB;    
                              read_back <= '1';    
                           ELSE
                              IF (((NOT clear_status_d(3) AND iG_INT2) OR read_idle_count(IDLE_MSB)) = '1') THEN
                                 p2s_data(15 DOWNTO 8) <= READ_MODE & INT_SOURCE;    
                                 clear_status <= '1';    
                              END IF;
                           END IF;
                        END IF;
                        IF ((high_byte OR read_ready OR read_idle_count(IDLE_MSB) OR (NOT clear_status_d(3) AND iG_INT2)) = '1') THEN
                           spi_go <= '1';    
                           spi_state <= TRANSFER;    
                        END IF;
                        IF (read_back_d = '1') THEN
                           IF (high_byte_d = '1') THEN
                              oDATA_H_xhdl2 <= s2p_data;    
                              oDATA_L_xhdl1 <= low_byte_data;    
                           ELSE
                              low_byte_data <= s2p_data;    
                           END IF;
                        END IF;
               WHEN TRANSFER =>
                        IF (spi_end = '1') THEN
                           spi_go <= '0';    
                           spi_state <= IDLE;    
                           IF (read_back = '1') THEN
                              read_back <= '0';    
                              high_byte <= NOT high_byte;    
                              read_ready <= '0';    
                           ELSE
                              clear_status <= '0';    
                              read_ready <= s2p_data(6);    
                              read_idle_count <= (OTHERS => '0');    
                           END IF;
                        END IF;
               WHEN OTHERS =>
                        NULL;
               
            END CASE;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (iRSTN, iSPI_CLK)
   BEGIN
      IF (iRSTN = '0') THEN
         high_byte_d <= '0';    
         read_back_d <= '0';    
         clear_status_d <= "0000";    
     ELSIF rising_edge(iSPI_CLK) THEN
          high_byte_d <= high_byte;    
         read_back_d <= read_back;    
         clear_status_d <= clear_status_d(2 DOWNTO 0) & clear_status;    
      END IF;
   END PROCESS;

END translated;
